# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class TerminalOptions(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'echo': 'int',
        'terminal': 'bool',
        'terminal_height': 'int',
        'terminal_type': 'str',
        'terminal_width': 'int',
        'ty_i_speed': 'int',
        'ty_o_speed': 'int'
    }

    attribute_map = {
        'echo': 'Echo',
        'terminal': 'Terminal',
        'terminal_height': 'TerminalHeight',
        'terminal_type': 'TerminalType',
        'terminal_width': 'TerminalWidth',
        'ty_i_speed': 'TyISpeed',
        'ty_o_speed': 'TyOSpeed'
    }

    def __init__(self, echo=None, terminal=None, terminal_height=None, terminal_type=None, terminal_width=None, ty_i_speed=None, ty_o_speed=None):  # noqa: E501
        """TerminalOptions - a model defined in Swagger"""  # noqa: E501

        self._echo = None
        self._terminal = None
        self._terminal_height = None
        self._terminal_type = None
        self._terminal_width = None
        self._ty_i_speed = None
        self._ty_o_speed = None
        self.discriminator = None

        if echo is not None:
            self.echo = echo
        if terminal is not None:
            self.terminal = terminal
        if terminal_height is not None:
            self.terminal_height = terminal_height
        if terminal_type is not None:
            self.terminal_type = terminal_type
        if terminal_width is not None:
            self.terminal_width = terminal_width
        if ty_i_speed is not None:
            self.ty_i_speed = ty_i_speed
        if ty_o_speed is not None:
            self.ty_o_speed = ty_o_speed

    @property
    def echo(self):
        """Gets the echo of this TerminalOptions.  # noqa: E501


        :return: The echo of this TerminalOptions.  # noqa: E501
        :rtype: int
        """
        return self._echo

    @echo.setter
    def echo(self, echo):
        """Sets the echo of this TerminalOptions.


        :param echo: The echo of this TerminalOptions.  # noqa: E501
        :type: int
        """

        self._echo = echo

    @property
    def terminal(self):
        """Gets the terminal of this TerminalOptions.  # noqa: E501


        :return: The terminal of this TerminalOptions.  # noqa: E501
        :rtype: bool
        """
        return self._terminal

    @terminal.setter
    def terminal(self, terminal):
        """Sets the terminal of this TerminalOptions.


        :param terminal: The terminal of this TerminalOptions.  # noqa: E501
        :type: bool
        """

        self._terminal = terminal

    @property
    def terminal_height(self):
        """Gets the terminal_height of this TerminalOptions.  # noqa: E501


        :return: The terminal_height of this TerminalOptions.  # noqa: E501
        :rtype: int
        """
        return self._terminal_height

    @terminal_height.setter
    def terminal_height(self, terminal_height):
        """Sets the terminal_height of this TerminalOptions.


        :param terminal_height: The terminal_height of this TerminalOptions.  # noqa: E501
        :type: int
        """

        self._terminal_height = terminal_height

    @property
    def terminal_type(self):
        """Gets the terminal_type of this TerminalOptions.  # noqa: E501


        :return: The terminal_type of this TerminalOptions.  # noqa: E501
        :rtype: str
        """
        return self._terminal_type

    @terminal_type.setter
    def terminal_type(self, terminal_type):
        """Sets the terminal_type of this TerminalOptions.


        :param terminal_type: The terminal_type of this TerminalOptions.  # noqa: E501
        :type: str
        """

        self._terminal_type = terminal_type

    @property
    def terminal_width(self):
        """Gets the terminal_width of this TerminalOptions.  # noqa: E501


        :return: The terminal_width of this TerminalOptions.  # noqa: E501
        :rtype: int
        """
        return self._terminal_width

    @terminal_width.setter
    def terminal_width(self, terminal_width):
        """Sets the terminal_width of this TerminalOptions.


        :param terminal_width: The terminal_width of this TerminalOptions.  # noqa: E501
        :type: int
        """

        self._terminal_width = terminal_width

    @property
    def ty_i_speed(self):
        """Gets the ty_i_speed of this TerminalOptions.  # noqa: E501


        :return: The ty_i_speed of this TerminalOptions.  # noqa: E501
        :rtype: int
        """
        return self._ty_i_speed

    @ty_i_speed.setter
    def ty_i_speed(self, ty_i_speed):
        """Sets the ty_i_speed of this TerminalOptions.


        :param ty_i_speed: The ty_i_speed of this TerminalOptions.  # noqa: E501
        :type: int
        """

        self._ty_i_speed = ty_i_speed

    @property
    def ty_o_speed(self):
        """Gets the ty_o_speed of this TerminalOptions.  # noqa: E501


        :return: The ty_o_speed of this TerminalOptions.  # noqa: E501
        :rtype: int
        """
        return self._ty_o_speed

    @ty_o_speed.setter
    def ty_o_speed(self, ty_o_speed):
        """Sets the ty_o_speed of this TerminalOptions.


        :param ty_o_speed: The ty_o_speed of this TerminalOptions.  # noqa: E501
        :type: int
        """

        self._ty_o_speed = ty_o_speed

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(TerminalOptions, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TerminalOptions):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from demisto_client.demisto_api.models.advance_arg import AdvanceArg  # noqa: F401,E501
from demisto_client.demisto_api.models.arg_filter import ArgFilter  # noqa: F401,E501
from demisto_client.demisto_api.models.task_condition import TaskCondition  # noqa: F401,E501


class InvTaskInfo(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'args': 'dict(str, AdvanceArg)',
        'comment': 'str',
        'conditions': 'list[TaskCondition]',
        'in_task_id': 'str',
        'input': 'str',
        'inv_id': 'str',
        'loop_args': 'dict(str, AdvanceArg)',
        'loop_condition': 'list[ArgFilter]',
        'version': 'int'
    }

    attribute_map = {
        'args': 'args',
        'comment': 'comment',
        'conditions': 'conditions',
        'in_task_id': 'inTaskID',
        'input': 'input',
        'inv_id': 'invId',
        'loop_args': 'loopArgs',
        'loop_condition': 'loopCondition',
        'version': 'version'
    }

    def __init__(self, args=None, comment=None, conditions=None, in_task_id=None, input=None, inv_id=None, loop_args=None, loop_condition=None, version=None):  # noqa: E501
        """InvTaskInfo - a model defined in Swagger"""  # noqa: E501

        self._args = None
        self._comment = None
        self._conditions = None
        self._in_task_id = None
        self._input = None
        self._inv_id = None
        self._loop_args = None
        self._loop_condition = None
        self._version = None
        self.discriminator = None

        if args is not None:
            self.args = args
        if comment is not None:
            self.comment = comment
        if conditions is not None:
            self.conditions = conditions
        if in_task_id is not None:
            self.in_task_id = in_task_id
        if input is not None:
            self.input = input
        if inv_id is not None:
            self.inv_id = inv_id
        if loop_args is not None:
            self.loop_args = loop_args
        if loop_condition is not None:
            self.loop_condition = loop_condition
        if version is not None:
            self.version = version

    @property
    def args(self):
        """Gets the args of this InvTaskInfo.  # noqa: E501


        :return: The args of this InvTaskInfo.  # noqa: E501
        :rtype: dict(str, AdvanceArg)
        """
        return self._args

    @args.setter
    def args(self, args):
        """Sets the args of this InvTaskInfo.


        :param args: The args of this InvTaskInfo.  # noqa: E501
        :type: dict(str, AdvanceArg)
        """

        self._args = args

    @property
    def comment(self):
        """Gets the comment of this InvTaskInfo.  # noqa: E501


        :return: The comment of this InvTaskInfo.  # noqa: E501
        :rtype: str
        """
        return self._comment

    @comment.setter
    def comment(self, comment):
        """Sets the comment of this InvTaskInfo.


        :param comment: The comment of this InvTaskInfo.  # noqa: E501
        :type: str
        """

        self._comment = comment

    @property
    def conditions(self):
        """Gets the conditions of this InvTaskInfo.  # noqa: E501


        :return: The conditions of this InvTaskInfo.  # noqa: E501
        :rtype: list[TaskCondition]
        """
        return self._conditions

    @conditions.setter
    def conditions(self, conditions):
        """Sets the conditions of this InvTaskInfo.


        :param conditions: The conditions of this InvTaskInfo.  # noqa: E501
        :type: list[TaskCondition]
        """

        self._conditions = conditions

    @property
    def in_task_id(self):
        """Gets the in_task_id of this InvTaskInfo.  # noqa: E501


        :return: The in_task_id of this InvTaskInfo.  # noqa: E501
        :rtype: str
        """
        return self._in_task_id

    @in_task_id.setter
    def in_task_id(self, in_task_id):
        """Sets the in_task_id of this InvTaskInfo.


        :param in_task_id: The in_task_id of this InvTaskInfo.  # noqa: E501
        :type: str
        """

        self._in_task_id = in_task_id

    @property
    def input(self):
        """Gets the input of this InvTaskInfo.  # noqa: E501


        :return: The input of this InvTaskInfo.  # noqa: E501
        :rtype: str
        """
        return self._input

    @input.setter
    def input(self, input):
        """Sets the input of this InvTaskInfo.


        :param input: The input of this InvTaskInfo.  # noqa: E501
        :type: str
        """

        self._input = input

    @property
    def inv_id(self):
        """Gets the inv_id of this InvTaskInfo.  # noqa: E501


        :return: The inv_id of this InvTaskInfo.  # noqa: E501
        :rtype: str
        """
        return self._inv_id

    @inv_id.setter
    def inv_id(self, inv_id):
        """Sets the inv_id of this InvTaskInfo.


        :param inv_id: The inv_id of this InvTaskInfo.  # noqa: E501
        :type: str
        """

        self._inv_id = inv_id

    @property
    def loop_args(self):
        """Gets the loop_args of this InvTaskInfo.  # noqa: E501


        :return: The loop_args of this InvTaskInfo.  # noqa: E501
        :rtype: dict(str, AdvanceArg)
        """
        return self._loop_args

    @loop_args.setter
    def loop_args(self, loop_args):
        """Sets the loop_args of this InvTaskInfo.


        :param loop_args: The loop_args of this InvTaskInfo.  # noqa: E501
        :type: dict(str, AdvanceArg)
        """

        self._loop_args = loop_args

    @property
    def loop_condition(self):
        """Gets the loop_condition of this InvTaskInfo.  # noqa: E501


        :return: The loop_condition of this InvTaskInfo.  # noqa: E501
        :rtype: list[ArgFilter]
        """
        return self._loop_condition

    @loop_condition.setter
    def loop_condition(self, loop_condition):
        """Sets the loop_condition of this InvTaskInfo.


        :param loop_condition: The loop_condition of this InvTaskInfo.  # noqa: E501
        :type: list[ArgFilter]
        """

        self._loop_condition = loop_condition

    @property
    def version(self):
        """Gets the version of this InvTaskInfo.  # noqa: E501


        :return: The version of this InvTaskInfo.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this InvTaskInfo.


        :param version: The version of this InvTaskInfo.  # noqa: E501
        :type: int
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(InvTaskInfo, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, InvTaskInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import unittest

from urllib3_mock import Responses
import demisto_client
from datetime import datetime
import json
responses = Responses('requests.packages.urllib3')

api_key = 'sample_api_key'
host = 'http://localhost:8080'


def assert_reset():
    assert len(responses._urls) == 0
    assert len(responses.calls) == 0


def test_get_docker_images():
    '''Check GET docker images.'''

    @responses.activate
    def run():
        body = '{ "images": [{"id": "aae7b8aaba8c", "repository": ' \
               '"openapitools/openapi-generator-cli", "tag": "latest", "createdSince": "3 days ago", ' \
               '"createdAt": "2019-08-19 13:34:22 +0300 IDT", "size": "131MB" }]}'
        responses.add('GET', '/settings/docker-images',
                      body=body,
                      status=200,
                      content_type='application/json')

        # create an instance of the API class
        api_instance = demisto_client.configure(base_url=host, api_key=api_key)
        api_response = api_instance.get_docker_images()

        assert api_response.images[0].created_at == '2019-08-19 13:34:22 +0300 IDT'
        assert api_response.images[0].id == 'aae7b8aaba8c'
        assert api_response.images[0].repository == 'openapitools/openapi-generator-cli'

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/settings/docker-images'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_create_incident():
    '''Check creating an incident.'''

    @responses.activate
    def run():
        body = '{"name":"Test Incident","owner":"Admin","parent":"","phase":"",' \
               '"playbookId":"playbook0","playbook_id":"playbook0","rawCategory":"",' \
               '"rawCloseReason":"","rawJSON":"","rawName":"Test Incident","rawPhase":"",' \
               '"rawType":"Unclassified","raw_category":"","raw_close_reason":"","raw_json":"",' \
               '"raw_name":"Test Incident","raw_phase":"","raw_type":"Unclassified","reason":"",' \
               '"runStatus":"","run_status":"","severity":0,"sourceBrand":"Manual",' \
               '"sourceInstance":"admin","source_brand":"Manual","source_instance":"admin",' \
               '"status":0,"type":"Unclassified","version":1}'
        responses.add('POST', '/incident',
                      body=body,
                      status=200,
                      content_type='application/json')

        # create an instance of the API class
        api_instance = demisto_client.configure(base_url=host, api_key=api_key)
        create_incident_request = demisto_client.demisto_api.CreateIncidentRequest()
        create_incident_request.name = 'Test Incident'
        create_incident_request.type = 'Unclassified'
        create_incident_request.owner = 'Admin'
        create_incident_request.occurred = datetime.now()
        api_response = api_instance.create_incident(create_incident_request=create_incident_request)

        assert api_response.name == 'Test Incident'
        assert api_response.type == 'Unclassified'
        assert api_response.owner == 'Admin'

        assert len(responses.calls) == 1
        req = responses.calls[0].request
        assert req.url == '/incident'
        assert req.host == 'localhost'
        assert req.scheme == 'http'
        # veriy date field occurred according to rfc 3339
        req_body = json.loads(req.body)
        assert req_body['occurred'][-6] == '+' or req_body['occurred'][-6] == '-'  # end with +/- offset 

    run()
    assert_reset()


def test_get_reports():
    '''Testing GET all reports.'''

    @responses.activate
    def run():
        body = '[{"created_by":"DBot","dashboard":"None","decoder":{},"description":"This report ' \
               'generates Mean Time to Resolve by Incident type for last 2 Quarters",' \
               '"id":"MTTRbyIncidentType2Quar","locked":false,"name":"Mean time to Resolve by ' \
               'Incident Type (Last 2 Quarters)","orientation":"portrait","prev_name":"Mean time to ' \
               'Resolve by Incident Type (Last 2 Quarters)","prev_type":"pdf","type":"pdf",' \
               '"version":4}]'

        responses.add('GET', '/reports',
                      body=body,
                      status=200,
                      content_type='application/json')

        api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)
        api_response = api_instance.get_all_reports()

        assert api_response[0].id == 'MTTRbyIncidentType2Quar'
        assert api_response[
                   0].description == 'This report generates Mean Time to Resolve by Incident type for ' \
                                     '' \
                                     'last 2 Quarters'
        assert api_response[0].version == 4

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/reports'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_indicators_search():
    '''Testing search for indicator.'''

    @responses.activate
    def run():
        body = r'''
        {
        "iocObjects": [{
            "id": "4737",
            "version": 1,
            "modified": "2019-07-14T16:54:02.719044+03:00",
            "account": "",
            "timestamp": "2019-07-14T16:54:02.718422+03:00",
            "indicator_type": "IP",
            "value": "92.63.197.153",
            "source": "Recorded Future",
            "investigationIDs": ["1750"],
            "lastSeen": "2019-07-14T16:54:02.718378+03:00",
            "firstSeen": "2019-07-14T16:54:02.718378+03:00",
            "lastSeenEntryID": "API",
            "firstSeenEntryID": "API",
            "score": 3,
            "manualScore": true,
            "setBy": "DBotWeak",
            "manualSetTime": "0001-01-01T00:00:00Z",
            "insightCache": null,
            "calculatedTime": "2019-07-14T16:54:02.718378+03:00",
            "lastReputationRun": "0001-01-01T00:00:00Z",
            "comment": "From Recorded Future risk list, Score - 89",
            "manuallyEditedFields": null
        }],
        "total": 1
    }
    '''

        responses.add('POST', '/indicators/search',
                      body=body,
                      status=200,
                      content_type='application/json')
        api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)
        indicator_filter = demisto_client.demisto_api.IndicatorFilter()  # IndicatorFilter |  (optional)
        indicator_filter.query = 'value:92.63.197.153'
        api_response = api_instance.indicators_search(indicator_filter=indicator_filter)

        assert api_response.ioc_objects[0].get('comment') == 'From Recorded Future risk list, Score - 89'
        # assert api_response.type == 'Unclassified'
        # assert api_response.owner == 'Admin'

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/indicators/search'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_export_entry():
    '''Testing export entry artifact.'''

    @responses.activate
    def run():
        body = "entry_artifact_6@1770.md"
        responses.add('POST', '/entry/exportArtifact',
                      body=body,
                      status=200,
                      content_type='application/json')
        api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)
        download_entry = demisto_client.demisto_api.DownloadEntry()  # DownloadEntry |  (optional)

        download_entry.id = '6@1770'
        download_entry.investigation_id = '1770'

        api_result = api_instance.entry_export_artifact(download_entry=download_entry)

        assert api_result == 'entry_artifact_6@1770.md'

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/entry/exportArtifact'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_generic_request():
    '''Testing generic requst.'''

    @responses.activate
    def run():
        responses.add('POST', '/test',
                      body="all good",
                      status=200,
                      content_type='text/plain')
        api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)
        (res, code, headers) = api_instance.generic_request('/test', 'POST', body="this is a test",
                                                            content_type='text/plain', accept='text/plain')

        assert res == 'all good'
        assert code == 200

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/test'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_import_layout(mocker):
    """
    Given:
        - Path for a layoutscontainer.
    When
        - Using client.import_layout() to upload a new layout.
    Then
        - The layout is being uploaded and getting back the layout metadata.
    """
    client = demisto_client.configure(base_url=host, api_key=api_key, debug=False,
                                      verify_ssl=False)
    mocker.patch.object(client, 'import_layout_with_http_info', return_value={'test': 'test'})
    res = client.import_layout('tests_data/layoutscontainer-test.json')
    assert res.get('test') == 'test'


def test_import_layout_with_http_info_with_knowing_server_version(mocker):
    """
    Given:
        - path for a layoutscontainer.
    When
        - Succeeding in getting demisto version using 'get_layouts_url_for_demisto_version' function
    Then
        - The layout is being uploaded and getting back the layout metadata.
    """
    client = demisto_client.configure(base_url=host, api_key=api_key, debug=False,
                                      verify_ssl=False)
    mocker.patch.object(client.api_client, 'call_api', side_effect=[("{'demistoVersion': '6.0.0'}", 200, {'Content-type': 'application/json'}),  {'test': 'test'}])
    res = client.import_layout('tests_data/layoutscontainer-test.json')
    assert res.get('test') == 'test'


def test_import_layout_with_http_info_without_knowing_server_version(mocker):
    """
    Given:
        - Path for a layoutscontainer.
    When
        - Not knowing demisto version, and working with 6.0.0 or higher
    Then
        - The layout is being uploaded and getting back the layout metadata.
    """
    client = demisto_client.configure(base_url=host, api_key=api_key, debug=False,
                                      verify_ssl=False)
    mocker.patch.object(client.api_client, 'call_api', side_effect=[("{'demistoVersion': '6.0.0'}", 404, {'Content-type': 'application/json'}),  {'test': 'test'}])
    res = client.import_layout('tests_data/layoutscontainer-test.json')
    assert res.get('test') == 'test'


def test_import_layout_with_http_info_with_old_layout_format(mocker):
    """
    Given:
        - Path for a layout
    When
        - Working with 5.0.0 and uploading a layout(the old version)
    Then
        - The layout is being uploaded and getting back the layout metadata.
    """
    client = demisto_client.configure(base_url=host, api_key=api_key, debug=False,
                                      verify_ssl=False)

    mocker.patch.object(client.api_client, 'call_api', side_effect=[("{'demistoVersion': '5.0.0'}", 200, {'Content-type': 'application/json'}),  {'test': 'test'}])
    res = client.import_layout('tests_data/layout-details-test-V2.json')
    assert res.get('test') == 'test'


class TestFailedGenericRequestNoEnv(unittest.TestCase):

    def test_generic_request(self):
        """
        Given:
        - Request which should result in an ApiException
        When:
        - No environment variable has been set
        Then:
        - Return ApiException without the headers in the error
        """
        from demisto_client.demisto_api.rest import ApiException

        @responses.activate
        def run():
            responses.add('POST', '/test',
                          body="Not good",
                          status=400,
                          content_type='text/plain')
            api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)

            with self.assertRaises(ApiException) as context:
                (_, _, _) = api_instance.generic_request('/test', 'POST',
                                                                    body="this is a test",
                                                                    content_type='text/plain',
                                                                    accept='text/plain')
            self.assertTrue('HTTP response body' in str(context.exception))
            self.assertFalse('HTTP response headers' in str(context.exception))
            assert len(responses.calls) == 1
            assert responses.calls[0].request.url == '/test'
            assert responses.calls[0].request.host == 'localhost'
            assert responses.calls[0].request.scheme == 'http'

        run()
        assert_reset()


class TestFailedGenericRequestWithEnv(unittest.TestCase):

    def test_generic_request(self):
        """
        Given:
        - Request which should result in an ApiException
        When:
        - Environment variable DEMISTO_EXCEPTION_HEADER_LOGGING has been set to true
        Then:
        - Return ApiException with the headers in the error
        """
        import sys
        # Error should be the same in both Py2 and Py3, but Py2 does not support unittest mock in
        # the same way
        if sys.version_info[0] > 2:
            import os
            from demisto_client.demisto_api.rest import ApiException
            from unittest import mock

            @mock.patch.dict(os.environ, {"DEMISTO_EXCEPTION_HEADER_LOGGING": "true"})
            @responses.activate
            def run():
                responses.add('POST', '/test',
                              body="Not good",
                              status=400,
                              content_type='text/plain')
                api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)

                with self.assertRaises(ApiException) as context:
                    (_, _, _) = api_instance.generic_request('/test', 'POST',
                                                             body="this is a test",
                                                             content_type='text/plain',
                                                             accept='text/plain')
                self.assertTrue('HTTP response body' in str(context.exception))
                self.assertTrue('HTTP response headers' in str(context.exception))
                assert len(responses.calls) == 1
                assert responses.calls[0].request.url == '/test'
                assert responses.calls[0].request.host == 'localhost'
                assert responses.calls[0].request.scheme == 'http'
        else:
            def run():
                assert 1 == 1
        run()
        assert_reset()
